/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the PubSub Sidebar.
 *
 * The Initial Developer of the Original Code is
 * PubSub Concepts, Inc.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
 
//========================================================
//
// the "server" for client sidebar windows.  
// 
// this code runs independently of the sidebar, so that 
// it can stay connected when the sidebar is closed.
// 
// ideally this should be application-global (at the 
// moment it's bound to a browser window)
//
//========================================================

/**
 * the max number of messages we're storing per subscription
 */
const MAX_BUCKET_SIZE = 32;

/**
 * how often we perform service operations (check the connection)
 * in seconds (we need millis for the function, though)
 */
const SERVICE_CHECK_TIMEOUT = 240; 

/**
 * default service host
 */
const DEFAULT_HOST = "xmpp.pubsub.com";

/**
 * default username
 */
const DEFAULT_USERNAME = "sample@pubsub.com";

/**
 * default password
 */
const DEFAULT_PASSWORD = "sample";

/** 
 * subscription buckets hold messages for each subscription
 */
var SubscriptionBuckets = new Array();

/** 
 * map of subscription IDs to subscription names
 */
var SubscriptionMap = new Array();

/**
 * set of keys: subscription IDs only
 */
var SubscriptionKeys = new Array();

/**
 * clients listening for new messages, subscriptions
 */
var ConnectedClients = new Array();

/** 
 * username: used for title 
 */
var ConnectedUser = "";

/**
 * last selected sub
 */
var LastSelectedSub = "";

/**
 * pending messages: a running count used for the toolbar button 
 * tooltip when the client sidebar window is closed
 */
var iPendingCount = 0;

/**
 * IDs used for locally identifying messages
 */
var iIDSequence = 100;

/**
 * cache error for clients.  code controls, and a code of 0 is no error.
 */
var LastErrorString = "";

/**
 * cache error for clients.  0 is no error.
 */
var LastErrorCode = 0;

/**
 * the name we connected with, not necessarily the 
 * current prefs name
 */
var ConnectedUsername = "";

/**
 * on startup, check if we want to connect automatically - 
 * and if that's the case, do so.
 */ 
function doAutoConnect()
{
	var parameters = null;
	try
	{
		parameters = getAccountPrefs();
	}
	catch( e )
	{
		parameters = setDefaultAccountPrefs();
	}
	
	try
	{
		if( parameters["auto"] == true )
		{
			doConnect( parameters );
		}
	}
	catch( e ){ alert( "E: " + e ); }
}

/**
 * default prefs...
 */
function setDefaultAccountPrefs()
{
	var d = new Date();
	var strTmp = "";
	var fTmp = false;
	var parameters = new Array();
	var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

	// set control flag...
	prefs.setBoolPref( "com.pubsub.pubsubsidebar.prefs_set", true );
	
	parameters["username"] = DEFAULT_USERNAME;
	parameters["password"] = DEFAULT_PASSWORD;
	parameters["host"] = DEFAULT_HOST;
	parameters["resource"] = "firefox-firsttime-" + d.getMilliseconds();
	
	// defaults for the bool prefs
	parameters["auto"] = true;
	parameters["showextended"] = true;
	parameters["closeboxes"] = false;
	parameters["closeonlink"] = false;
	parameters["loadhistory"] = true;
	parameters["showdates"] = false;

	prefs.setCharPref( "com.pubsub.pubsubsidebar.username", parameters["username"]  );
	prefs.setCharPref( "com.pubsub.pubsubsidebar.password", parameters["password"]  );
	prefs.setCharPref( "com.pubsub.pubsubsidebar.host", parameters["host"]  );
	
	prefs.setBoolPref( "com.pubsub.pubsubsidebar.autoconnect", parameters["auto"] );
	prefs.setBoolPref( "com.pubsub.pubsubsidebar.showextended", parameters["showextended"] );
	prefs.setBoolPref( "com.pubsub.pubsubsidebar.closeboxes", parameters["closeboxes"] );
	prefs.setBoolPref( "com.pubsub.pubsubsidebar.loadhistory", parameters["loadhistory"] );
	prefs.setBoolPref( "com.pubsub.pubsubsidebar.closeonlink", parameters["closeonlink"] );
	prefs.setBoolPref( "com.pubsub.pubsubsidebar.showdates", parameters["showdates"] );

	return parameters;
}

/**
 * get account settings from prefs
 */
function getAccountPrefs()
{
	var d = new Date();
	var strTmp = "";
	var fTmp = false;
	var parameters = new Array();
	var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
	
	// start with defaults
	parameters["host"] = DEFAULT_HOST;

	// some of these are new, and may throw exceptions if not found; that's 
	// ok, but we do want to throw an exception if prefs have never been set.
	
	fTmp = prefs.getBoolPref( "com.pubsub.pubsubsidebar.prefs_set" );
	
	// if this is false, but doesn't throw an exception, handle it anyway
	if( !fTmp ) return setDefaultAccountPrefs();

	try
	{
		fTmp = prefs.getBoolPref( "com.pubsub.pubsubsidebar.autoconnect" );
		parameters["auto"] = fTmp;
	}
	catch( e ){ parameters["auto"] = true; }
	
	try
	{
		fTmp = prefs.getBoolPref( "com.pubsub.pubsubsidebar.showextended" );
		parameters["showextended"] = fTmp;
	}
	catch( e ){ parameters["showextended"] = true; }
		
	try
	{
		fTmp = prefs.getBoolPref( "com.pubsub.pubsubsidebar.closeboxes" );
		parameters["closeboxes"] = fTmp;
	}
	catch( e ){ parameters["closeboxes"] = false; }
	
	try
	{
		fTmp = prefs.getBoolPref( "com.pubsub.pubsubsidebar.loadhistory" );
		parameters["loadhistory"] = fTmp;
	}
	catch( e ){ parameters["loadhistory"] = true; }

	try
	{
		fTmp = prefs.getBoolPref( "com.pubsub.pubsubsidebar.closeonlink" );
		parameters["closeonlink"] = fTmp;
	}
	catch( e ){ parameters["closeonlink"] = false; }

	try
	{
		fTmp = prefs.getBoolPref( "com.pubsub.pubsubsidebar.showdates" );
		parameters["showdates"] = fTmp;
	}
	catch( e ){ parameters["showdates"] = true; }

	strTmp = prefs.getCharPref( "com.pubsub.pubsubsidebar.username" );
	if( null != strTmp ) parameters["username"] = strTmp;
	
	strTmp = prefs.getCharPref( "com.pubsub.pubsubsidebar.password" );
	if( null != strTmp ) parameters["password"] = strTmp;

	strTmp = prefs.getCharPref( "com.pubsub.pubsubsidebar.host" );
	if( null != strTmp ) parameters["host"] = strTmp;

	parameters["resource"] = "firefox-" + d.getMilliseconds();
	return parameters;
}

/**
 * register a client
 */
function registerClient( client )
{
	var i = 0;
	var found = false;

	// make sure it's not already in the list
	for( i = 0; !found && i< ConnectedClients.length; i++)
	{
		if( ConnectedClients[i] == client ) found = true;
	}
	
	// if not, push on the end
	if( !found )
	{
		ConnectedClients[ConnectedClients.length] = client; 
		// alert( "Registering client: " + client );
	}
	
	// send the last error to this client so it can
	// display it, if necessary.
	
	onJabberError( LastErrorCode, LastErrorString );
}

function refWaitCursor( amt )
{
	var i = 0;
	for( i = 0; i< ConnectedClients.length; i++)
	{
		try{ ConnectedClients[i].refWaitCursor( amt ); }
		catch( e ) { /* alert( "e: " + e ); */ }
	}
}

/**
 * unregister as a client 
 */
function unregisterClient( client )
{
	var i = 0;
	var found = false;
	for( i = 0; i< ConnectedClients.length; i++)
	{
		if( !found )
			if( ConnectedClients[i] == client ) found = true;
		if( found )
		{
			if( i + 1 < ConnectedClients.length )
				ConnectedClients[i] = ConnectedClients[i+1];
		}
	}
	if( found ) ConnectedClients.pop();
}

/**
 * are we currently connected? 
 */
function isConnected()
{
	return ( jabberComponent.connectionState != 0 
			 && jabberComponent.connectionState != 100 );
}

/**
 * check connection; try to reconnect, if necessary
 */
function checkAndReconnect()
{
	logText( "checkAndReconnect" );
	
	if( !isConnected()
		&& jabberComponent.fReconnect )
	{
		logText( "Reconnecting..." );
		jabberComponent.startJabber();
	}
}

/**
 * send a ping packet to the server.  this is done so that if the socket
 * was silently dropped, we'll get an exception and close it.  once it's
 * closed, the reconnect function should be able to restart it.
 */
function pingServer()
{
	logText( "calling ping server" );
	if( isConnected()) jabberComponent.sendPingPacket();
}

/**
 * this function happens every X seconds, to check connection state
 * and ping the server (if necessary).  it doesn't need to happen that
 * often.
 */
function serviceCheck()
{
	logText( "serviceCheck" );

	checkAndReconnect();
	pingServer();
	setTimeout( "serviceCheck()", SERVICE_CHECK_TIMEOUT * 1000 );
}

/**
 * reset application variables (in particular, buckets)
 */
function resetAll()
{
	SubscriptionBuckets = new Array();
	SubscriptionKeys = new Array();
	SubscriptionMap = new Array();
	clearError();
}

/** 
 * push a message into a subscription bucket 
 */
function pushInBucket( subID, msg )
{
	// about declaring locals: if you don't, the parser
	// will use a global, which can interfere with a
	// similarly-named local in a calling function and 
	// screw things up.
	
	var i = 0;
	var arr = null;
	var max = 0;

	// normalize
	if( subID == null ) subID = "";
		
	// get the bucket for this sub
	arr = SubscriptionBuckets[subID];
	if( null == arr ) arr = new Array();
	
	// roll the array
	max = arr.length;
	if( max > MAX_BUCKET_SIZE ) max = MAX_BUCKET_SIZE;
	for( i = max; i > 0; i-- ) arr[i] = arr[i-1];
	
	// and push the new one on top
	arr[0] = msg;
	
	// store the bucket
	SubscriptionBuckets[subID] = arr;
	
	//logText( "bucket " + subID + " size is " + arr.length );
}

function logText( str )
{
	var i = 0;
	for( i = 0; i< ConnectedClients.length; i++)
	{
		try{ ConnectedClients[i].onLogText( str ); }
		catch( e ) { /* alert( "e: " + e ); */ }
	}
}

/**
 * app function: clear error
 */
function clearError()
{
	onJabberError(0, null);
}

/**
 * callback method for errors; call clients...
 */
function onJabberError( code, str )
{
	logText( "ERR [" + code + "]\n" + str );

	// we may be able to map this error to a longer string...
	switch( code )
	{
	case -1:
		str = "Login failed.<BR/><BR/>Please check your username and password on the options screen and try again.";
		break;
	case 2:
		str = "A connection error occured.  Please try again later.";
		break;
	case 4:
		str = "A connection error occured.  Please try again later.";
		break;
	}

	LastErrorString = str;
	LastErrorCode = code;

	// call listeners...
	var i = 0;
	for( i = 0; i< ConnectedClients.length; i++)
	{
		try{ ConnectedClients[i].onError( code, str ); }
		catch( e ) { /* alert( "e: " + e ); */ }
	}
	
}

/**
 * callback method for state changes
 */
function onStateChange( newState )
{
	// call listeners...
	var i = 0;
	for( i = 0; i< ConnectedClients.length; i++)
	{
		try{ ConnectedClients[i].onStateChange( newState ); }
		catch( e ) { /* alert( "e: " + e ); */ }
	}
	
}

/**
 * get all subs - useful for a reconnect from a client 
 * to a running server
 */
function getAllSubs( client )
{
	var i = 0;
	client.onResetSubs();
	
	for( i = 0; i< SubscriptionKeys.length; i++ ) 
		client.onNewSub( SubscriptionMap[SubscriptionKeys[i]] );

}

/** 
 * callback method: will be called on new subscription, so
 * we need to add it to the dropdown list.
 */
function onNewSub( sub )
{
	//logText( "onNewSub (" + sub.ID + ")" );

	// map local
	SubscriptionMap[sub.ID] = sub;
	SubscriptionKeys.push( sub.ID );

	//logText( "{check: " + SubscriptionMap.length + "}" );
	//logText( "{check2: " + SubscriptionMap[sub.ID] + "}" );

	// call listeners...

	var i = 0;
	for( i = 0; i< ConnectedClients.length; i++)
	{
		try{ ConnectedClients[i].onNewSub( sub ); }
		catch( e ) { /* alert( "e: " + e ); */ }
	}
	
}

/**
 * callback method: when a new "affiliations" response is
 * received, we dump old subs and rebuild the list.  this
 * prevents doubling up.
 */
function onResetSubs()
{
	// call listeners...

	var i = 0;
	for( i = 0; i< ConnectedClients.length; i++)
	{
		try{ ConnectedClients[i].onResetSubs( ); }
		catch( e ) { /* alert( "e: " + e ); */ }
	}
}

function onSubListComplete()
{
	/*
	var subsListObj = FindSubsList();
	if( null != subsListObj && subsListObj.length > 0 )
	{
		subsListObj.selectedIndex = 1;
		doUpdateSub();
	}
	*/
	var i = 0;
	for( i = 0; i< ConnectedClients.length; i++)
	{
		try{ ConnectedClients[i].onSubListComplete( ); }
		catch( e ) { /* alert( "e: " + e ); */ }
	}
}

/**
 * reset the pending counter: called when a client opens, so 
 * we stop accumulating "new" messages
 */
function resetPendingCount()
{
	iPendingCount = 0;
}

function deleteID( msgID, bucket )
{
	// delete the message represented by ID.  bucket is
	// offered as a locating aid.  it's almost certainly
	// correct so, for now, we won't bother with the
	// error condition.
	
	var i = 0;
	var found = false;
	var arr = SubscriptionBuckets[ bucket ];
	if( null == arr ) return;

	// NOTE ALSO that we're only removing the message in
	// one place.  That may or may not be what you want...

	for( i = 0; i< arr.length; i++ )
	{
		if( found )
		{
			arr[i-1] = arr[i];
		}
		else if( arr[i].messageID == msgID )
		{
			found = true;
		}
	}
	if( found ) arr.pop();

	// put back

	SubscriptionBuckets[ bucket ] = arr;

}

/** 
 * callback method: will be called on new message arrival 
 */
function onNewMessage( msg )
{
	// check if we need to display this one
	var i = 0;
	var iClientsSet = 0;
	
	// push the subscription into appropriate buckets
	for( i = 0; i< msg.subIDs.length; i++ )
	{
		pushInBucket( msg.subIDs[i], msg );
	}

	// push into the "new" bucket
	pushInBucket( "", msg );
	
	// notify listeners...

	for( i = 0; i< ConnectedClients.length; i++)
	{
		try
		{ 
			ConnectedClients[i].onNewMessage( msg ); 
			iClientsSet++;
		}
		catch( e ) 
		{ 
		}
	}
	
	if( iClientsSet == 0 )
	{
		// set tb button to hot style
		var obj = document.getElementById( "pubsubsidebar-button" );
		if( null != obj ) obj.className = "highlighted";
		
		// set tooltip
		iPendingCount++;
		if( null != obj )
		{
			var str = "PubSub Sidebar: " +
					  iPendingCount + " new message";
			if( iPendingCount > 1 ) str += "s" ;
			obj.tooltipText = str; 
		}
	}
	else iPendingCount = 0;
}

/**
 * app function: set up data, then open the jabber connection
 */
function doConnect( parameters )
{
	/*
	alert( "DoConnect()" );
	alert( "u: " + parameters["username"] );
	alert( "p: " + parameters["password"] );
	alert( "h: " + parameters["host"] );
	alert( "r: " + parameters["resource"] );
	*/

	// reset data
	resetAll();

	// get login stuff
	//getConnectionParameters( jabberComponent );

	jabberComponent.username = translateUsername( parameters["username"] );
	jabberComponent.password = parameters["password"];
	jabberComponent.host = parameters["host"];
	jabberComponent.connectionResource = parameters["resource"];

	// set callbacks
	jabberComponent.fnNewSubscription = onNewSub;	
	jabberComponent.fnNewMessage = onNewMessage;	
	jabberComponent.fnResetSubscriptions = onResetSubs;
	jabberComponent.fnError = onJabberError;
	jabberComponent.fnSubListComplete = onSubListComplete;
	jabberComponent.fnStateChange = onStateChange;

	// save the user for display
	ConnectedUsername = parameters["username"];

	// connect	
	jabberComponent.startJabber();
	
	// save user info
	// storeUserPrefs();
}

/**
 * app function: close the jabber connection
 */
function doCloseConnection()
{
	// unset flag...
	jabberComponent.fReconnect = false;
	
	// close jabber gracefully
	jabberComponent.closeConnection();
}

/** 
 * translate user to a xmpp.pubsub.com address, do
 * substitutions
 */
function translateUsername( username )
{
	username = username.replace( /@/, "_at_" );
	username = username.replace( /\+/g, "_plus_" );
	username = username.replace( /\./g, "_dot_" );
	username += "@xmpp.pubsub.com";
	return username;
}

/**
 * class prototype for the subscription object
 */
function PubsubSubscriptionType()
{
	this.ID = "";
	this.name = "";
	this.keywords = "";
	this.historyFetched = false;
	this.messageCount = 0;
}

/**
 * class prototype for the message object
 */
function PubsubMessageType()
{
	this.title = "";
	this.source = "";
	this.subIDs = new Array();
	this.body = "";
	this.link = "";
	this.sourceLink = "";
	this.date = "";
	this.note = "";
	this.messageID = iIDSequence++;
}

/**
 * utility function for dealing with cdata blocks
 */
function scrubCDATA( str )
{
	var cdataRE = /\<!\[CDATA\[(.+?)\]\]\>/;
	var match = cdataRE.exec( str );
	if( null == match ) return str;

	str = match[1];
	str.replace( /\</g, "&lt;" );
	str.replace( /\>/g, "&gt;" );
	str.replace( /\&/g, "&amp;" );
	
	return str;
}

function ensureSubHistory( subID )
{
	logText( "ESH: " + subID );

	// maybe we need to load history here...
	if( "" != subID )
	{
		var subObj = SubscriptionMap[subID];
	
		if( null != subObj )
		{
			if( subObj.historyFetched == false )
			{
				var arr = SubscriptionBuckets[ subID ];
				
				// set flag
				subObj.historyFetched = true;

				// store, jic, I'm not sure how the references in js work
				SubscriptionMap[subID] = subObj;

				// if the count in the sub is already > max, then
				// we can skip this part.  otherwise, load the history.
				
				if( null == arr || arr.length < MAX_BUCKET_SIZE )
				{
					logText( "Calling GSH for sub " + subID );
					getSubHistory( subID );
					logText( "GSH Call Returned" );
				}
				
			}
		}
	}
}

/**
 * retrieve the history for a subscription
 */
function getSubHistory( subID )
{
	var listener = 
	{
		linkElementRE: /<link\s.*?\>/g, 
		psLinkElementRE: /<link\s.*?\>/g, 
		getHREFRE: /href=(?:'|")(.*?)(?:"|')/,
		entryElementRE: /<entry(?:\s|\>).+?\<\/entry\>/g,
		checkHTMLType: /type=(?:'|")text\/html(?:"|')/,
		titleRE2: /<title>(.*?)<\/title>/,
		sourceTitleElementRE: /<(?:ps.title|title).*?\>(.+?)\<\/(?:ps.title|title)\>/,
		sourceFeedRE: /<(?:ps.source-feed|source-feed)(?:\s|\>).+\<\/(?:ps.source-feed|source-feed)\>/,
		modifiedRE: /<modified>(.+?)<\/modified>/,
		
		subID: "",
	
		finished : function(data)
		{
			logText( "History request complete" );
			
			var tmp = null;
			var entry = null;
			var sourceFeed = null;
			var entryString = "";
			var messageArray = new Array();
			var link = null;
			var source = null;
			var linkStr = "";
			var lastLinkStr = "";
			var title = null;
			
			// get the sub obj, we need it for keywords
			//var subObj = SubscriptionMap[this.subID];
			//var keywords = "";
			//if( null != subObj ) keywords = subObj.keywords;
						
			while( true )
			{
				entry = this.entryElementRE.exec( data );
				if( null == entry ) break;
				
				//logText( "------------" );
				
				var msg = new PubsubMessageType();
				
				entryString = entry[0];
				sourcefeed = this.sourceFeedRE( entry[0] );
				if( null != sourcefeed )
				{	
					//logText( "SF == " + sourcefeed[0] );
				
					// get useful stuff here
					source = this.sourceTitleElementRE.exec( sourcefeed[0] );
					if( null != source )
					{
						//logText( "Source: " + source[1] );
						msg.source = scrubCDATA( source[1] );
					}

					linkStr = "";
					lastLinkStr = "";
					do
					{
						link = this.psLinkElementRE.exec( sourcefeed[0] );
						//logText( "SF LINK: " + link );
						
						if( null != link )
						{
							tmp = this.getHREFRE.exec( link[0] );
							lastLinkStr = tmp[1];
							if( null != this.checkHTMLType( link[0] )) linkStr = tmp[1];
						}
					}
					while( null != link );
					if( linkStr == "" ) linkStr = lastLinkStr;
					
					msg.sourceLink = linkStr;

					// remove the source-feed from the entry string
					entryString = entryString.replace( this.sourceFeedRE, "" );
				}
				
				msg.title = "Untitled Entry";
				title = this.titleRE2.exec( entryString );
				if( null != title )
					msg.title = scrubCDATA( title[1] );
	
				msg.date = "(undated)";
				tmp = this.modifiedRE.exec( entryString );
				if( null != tmp ) msg.date = tmp[1];

				//logText( "Title: " + msg.title );
				
				linkStr = "";
				lastLinkStr = "";
				while( true )
				{
					link = this.linkElementRE.exec( entryString );
					if( null == link ) break;
					tmp = this.getHREFRE.exec( link[0] );
					lastLinkStr = tmp[1];
					if( null != this.checkHTMLType( link[0] )) linkStr = tmp[1];
				}
				if( linkStr == "" ) linkStr = lastLinkStr;
				msg.link = linkStr;
				//msg.keywords = keywords;
				msg.subIDs[0] = this.subID;
				
				// push
				messageArray[ messageArray.length ] = msg;
				
			}			
			
			// now push into the array for the sub
			var bucket = SubscriptionBuckets[this.subID];
			if( null == bucket ) bucket = new Array();
			
			var i = 0;
			
			// NOTE: we're going to skip however many messages are already
			// in the list.  we can assume there are dupes from recent matches,
			// if any are present.  This might result in too many being removed,
			// so we need to change this to a comparison method, but this 
			// will work for the time being.
			
			for( i = bucket.length; i< messageArray.length; i++)
			{
				bucket[ bucket.length ] = messageArray[i];
				if( bucket.length >= MAX_BUCKET_SIZE ) break;
			}
			
			// store the bucket
			SubscriptionBuckets[this.subID] = bucket;
			
			/*
			logText( "Repainting..." );
			
			// now repaint the sub 
			if( CurrentBucket == this.subID )
			{
				// set to blank so it'll force a repaint
				CurrentBucket = "";
				doDisplaySub( this.subID );
			}
			*/
			
			logText( "Notifying listeners" );
			
			// notify listeners...
			var ii = 0;
			for( ii = 0; ii< ConnectedClients.length; ii++)
			{
				try{ ConnectedClients[ii].onSubHistoryComplete( subID ); }
				catch( e ) { /* alert( "e: " + e ); */ }
			}
		
		}
	}

	// construct the request.  get the pubsub/subs/ out of the sub ID.  make
	// sure to use http/1.1 and pass the Host: header.  Also, moz doesn't seem
	// to like keepalive here, so pass Connection: close.
	
	// update: add the style=summary to kick out descriptions, we don't need them.
	
	var strRequest = "GET /" + subID + ".xml&style=summary HTTP/1.1\n" + 
					 "Host: atom.pubsub.com\n" + 
					 "Connection: close\n" +
					 "\n";
	strRequest = strRequest.replace( /pubsub\/subs\//, "" );
	listener.subID = subID;

	try 
	{
		var transportService = Components.classes["@mozilla.org/network/socket-transport-service;1"].getService(Components.interfaces.nsISocketTransportService);
		var transport = transportService.createTransport( null, 0, "atom.pubsub.com", 80, null);
		var outstream = transport.openOutputStream( 0, 0, 0 );
				
		logText( "Sending request:" );
		logText( strRequest );
		
		outstream.write( strRequest, strRequest.length );

		var stream = transport.openInputStream( 0,0,0 );
		var instream = Components.classes["@mozilla.org/scriptableinputstream;1"].createInstance(Components.interfaces.nsIScriptableInputStream);
		instream.init( stream );

		var dataListener = 
		{
			data : "",
			onStartRequest: function(request, context)
			{
				refWaitCursor( 1 );
			},
			onStopRequest: function(request, context, status)
			{

				refWaitCursor( -1 );

				instream.close();
				outstream.close();
				listener.finished(this.data);
			},
			onDataAvailable: function(request, context, inputStream, offset, count)
			{

				var strData = instream.read(count);
				strData = strData.replace( /(?:\n|\r)/g, " " );
				this.data += strData;
			},
		};

		var pump = Components.classes["@mozilla.org/network/input-stream-pump;1"].createInstance(Components.interfaces.nsIInputStreamPump);
		pump.init(stream, -1, -1, 0, 0, false);
		pump.asyncRead(dataListener,null);
		
	}
	catch(e)
	{
		//logText( "Exception in fetch history: " + e );
		return e;
	}
	return null;
}

//////////////////////


/** 
 * class definition for jabber object 
 */
var	jabberComponent = 
{
	//===============================================
	//
	// members
	//
	//===============================================
	
	host : "",
	data : "",
	outstream : null,
	instream : null,
	username : "",
	password : "",
	port : 5222,
	connectionResource : "",
	connectionState : 0,
	fReconnect : false,
	startTime : 0,
	stopTime : 0,

	//===============================================
	//
	// app callback functions
	//
	//===============================================
	
	// prototype: fnNewSubscription( PubsubSubscriptionType )
	fnNewSubscription: null,
	
	// prototype: fnNewMessage( PubsubMessageType )
	fnNewMessage: null,
	
	// prototype: fnError( ErrorCode, ErrorString )
	fnError: null,
	
	// prototype: fnResetSubscriptions()
	fnResetSubscriptions: null,
	
	// prototype: fnStateChange( newState )
	fnStateChange: null,
	
	// prototype: fnSubListComplete()
	fnSubListComplete: null,
	
	//===============================================
	//
	// regular expressions
	//
	//===============================================
	
	longIQRE: /\<iq.+\<\/iq\>/,
	linkElementRE: /<link\s.*?\>/g, 
	singleTagIQRE: /\<iq\s[^\>]+?\/\>/,
	psLinkElementRE: /<link\s.*?\>/g, 
	getHREFRE: /href=(?:'|")(.*?)(?:"|')/,
	openStreamRE: /\<stream(?:\:|\s).*?\>/, 
	packetIDRE: /\sid=(?:'|")(.+?)(?:"|')/,
	modifiedRE: /<modified>(.+?)<\/modified>/,
	packetTypeRE: /\stype=(?:'|")(.+?)(?:"|')/,
	feedElementRE: /<feed(?:\s|\>).+\<\/feed\>/,
	closeStreamRE: /\<\/stream(?:\:stream\>|\>)/, 
	messageRE: /\<message(?:\s|\>).+\<\/message\>/,
	entryElementRE: /<entry(?:\s|\>).+\<\/entry\>/,
	checkHTMLType: /type=(?:'|")text\/html(?:"|')/,
	headerElementRE: /<header\s.*?\>(.+?)\<\/header\>/g,
	headersElementRE: /<headers\s.*?\>(.+?)\<\/headers\>/,
	affiliationsRE: /<affiliations.*?\>(.+?)\<\/affiliations\>/,
	pubsubMessageRE: /<pubsub-message(?:\s|\>).+\<\/pubsub-message\>/,
	subTitleRE: /<x .*?\<field var=.title.>\s*\<value\>(.*?)<\/value/,
	entityRE: /<entity.*?subid=(?:'|")(.+?)(?:"|').*?\>(.+?)\<\/entity\>/g,
	titleElementRE: /<(?:ps.title|title).*?\>(.+?)\<\/(?:ps.title|title)\>/,
	subKeywordsRE: /<x .*?\<field var=.query-keywords.>\s*\<value\>(.*?)<\/value/,
	sourceTitleElementRE: /<(?:ps.title|title).*?\>(.+?)\<\/(?:ps.title|title)\>/,
	sourceFeedRE: /<(?:ps.source-feed|source-feed)(?:\s|\>).+\<\/(?:ps.source-feed|source-feed)\>/,

	//===============================================
	//
	// methods
	//
	//===============================================
	
	/** 
	 * initialize the _jabber_ connection.  the socket must be open. 
	 * we're sending the open stream and the auth packet at the same time.
	 */
	initConnection: function()
	{
		logText( "Calling initConnection" );
				
		var initPacket = 
			"<stream:stream to='" + this.host + "' xmlns='jabber:client' xmlns:stream='http://etherx.jabber.org/streams'>\n" +
			"<iq type='set' id='auth_packet'>\n" +
			"<query xmlns='jabber:iq:auth'>\n" +
			"<username>" + this.username + "</username>\n" +
			"<password>" + this.password + "</password>\n" +
			"<resource>" + this.connectionResource + "</resource>\n" +
			"</query>\n" +
			"</iq>\n";
		
		logText( "Packet\n------\n" + initPacket + "\n-------\n" );
				
		this.outstream.write( initPacket, initPacket.length );
	},
	
	/** 
	 * close the jabber connection: send a close packet
	 */
	closeConnection: function()
	{
		logText( "closeConnection()" );
		
		this.fReconnect = false;
		
		var closePacket = 
			"</stream:stream>\n";
		this.outstream.write( closePacket, closePacket.length );
		this.updateState( 100 );

	},

	/**
	 * send a small packet to test the connection
	 */
	sendPingPacket: function()
	{
		var pingPacket = "\n";
		this.outstream.write( pingPacket, pingPacket.length );
	},

	/** 
	 * read the stream; look for known elements and, if found, process them. 
	 */
	processDataStream: function()
	{
		logText( "packet" );
		// alert( "packet\n" + this.data );	
	
		// check for known packets
		
		var keepMatching = false;
		var matches = null;
		var strTmp = "";
				
		do
		{
			keepMatching = false;
			
			matches = this.messageRE.exec( this.data );
			if( null != matches )
			{
				//this.data = this.data.substr( matches.index + matches[0].length );
				
				strTmp = this.data.substr( 0, matches.index );
				strTmp += this.data.substr( matches.index + matches[0].length );
				this.data = strTmp;
				
				this.processMessagePacket( matches[0] );
				keepMatching = true;
			}

			matches = this.singleTagIQRE.exec( this.data );
			if( null != matches )
			{
				//this.data = this.data.substr( matches.index + matches[0].length );
				
				strTmp = this.data.substr( 0, matches.index );
				strTmp += this.data.substr( matches.index + matches[0].length );
				this.data = strTmp;
				
				this.processIQPacket( matches[0] );
				keepMatching = true;
			}
	
			matches = this.longIQRE.exec( this.data );
			if( null != matches )
			{
				//this.data = this.data.substr( matches.index + matches[0].length );
				
				strTmp = this.data.substr( 0, matches.index );
				strTmp += this.data.substr( matches.index + matches[0].length );
				this.data = strTmp;
				
				this.processIQPacket( matches[0] );
				keepMatching = true;
			}
	
			matches = this.openStreamRE.exec( this.data );
			if( null != matches )
			{
				//logText( "got open stream packet" );
				//this.data = this.data.substr( matches.index + matches[0].length );
				
				strTmp = this.data.substr( 0, matches.index );
				strTmp += this.data.substr( matches.index + matches[0].length );
				this.data = strTmp;
				
				keepMatching = true;
			}
		
			matches = this.closeStreamRE.exec( this.data );
			if( null != matches )
			{
				//logText( "got close stream packet" );
				//this.data = this.data.substr( matches.index + matches[0].length );
				
				strTmp = this.data.substr( 0, matches.index );
				strTmp += this.data.substr( matches.index + matches[0].length );
				this.data = strTmp;
				
				keepMatching = false;
			}
			
			// if( !keepMatching ) logText( "No match on data..." );// " + this.data );
			
		}
		while( keepMatching && this.data.length > 0 );
				
	},

	/**
	 * process an IQ packet.  at the moment we're interested in 
	 * auth packets and affiliations results packets.
	 */
	processIQPacket: function( packet )
	{
		logText( "{ IQ Packet }" );
		
		var entity = null;
		var subInfo = null;
		var subKWInfo = null;
		var match = this.affiliationsRE.exec( packet );
		
		var packetID = "";
		var packetType = "";
		
		if( null != match )
		{
			if( null != this.fnResetSubscriptions ) this.fnResetSubscriptions();
			do
			{
				entity = this.entityRE.exec( match[0] );
				if( null != entity )
				{
					// the sub ID is in 1; the contents in 2
					subInfo = this.subTitleRE.exec( entity[2] );
					subKWInfo = this.subKeywordsRE.exec( entity[2] );
					if( null != subInfo )
					{
						//logText( "subID: " + entity[1] );
						//logText( "Name: " + subInfo[1] );
						//logText( "" );
				
						var subObj = new PubsubSubscriptionType();
						subObj.ID = entity[1];
						subObj.name = subInfo[1];
						
						if( null != subKWInfo ) subObj.keywords = subKWInfo[1];
					
						// callback function for app
						if( null != this.fnNewSubscription ) this.fnNewSubscription( subObj );
					}
					else
					{
						//logText( "** WARNING: couldn't get sub name for subID: " + entity[1] );
					}
				}
			}
			while( entity != null );
				
			if( null != this.fnSubListComplete ) this.fnSubListComplete();

			return;
		}
		
		match = this.packetIDRE.exec( packet );
		if( null != match ) packetID = match[1];
		
		match = this.packetTypeRE.exec( packet);
		if( null != match ) packetType = match[1];

		if( packetID == "auth_packet" )
		{
			if( packetType == "result" )
			{
				this.updateState( 10 ); // login ok
				this.sendAffiliations();
			}
			else
			{
				this.updateState( 9 ); // login failed
				logText( packet );
				
				if( null != this.fnError ) this.fnError( -1, "Login failed" );
				
				// on login error, cancel reconnect
				this.fReconnect = false;
				
				// we can't continue here
				this.closeConnection();

			}
			return;
		}

	},

	/**
	 * open the connection, and if that succeeds, start the jabber stream.
	 */
	startJabber: function()
	{
		if( null == this.connectToSocket())
		{
			// socket connect succeeded; start the jabber stream
			this.initConnection();
			
			// set reconnect flag
			this.fReconnect = true;
		}
		else
		{
			//logText( "Err: connect to socket failed" );
			if( null != fnError ) fnError( 2, "Socket connection failed." );
			
			// set reconnect flag
			this.fReconnect = false;

		}
	},

	/**
	 * create the socket connection and open communication streams;
	 * set up for async communication with the server.
	 */
	connectToSocket: function()
	{
		try 
		{		
			//logText( "Connecting streams" );

			var	transportService = Components.classes["@mozilla.org/network/socket-transport-service;1"].getService(Components.interfaces.nsISocketTransportService);
			var	transport =	transportService.createTransport( null, 0, this.host, this.port, null );
			this.outstream = transport.openOutputStream( 0, 0, 0 ); 

			var	stream = transport.openInputStream(0,0,0);
			this.instream = Components.classes["@mozilla.org/scriptableinputstream;1"].createInstance(Components.interfaces.nsIScriptableInputStream);
			this.instream.init( stream );

			var	pump = Components.classes["@mozilla.org/network/input-stream-pump;1"].createInstance(Components.interfaces.nsIInputStreamPump);
			pump.init( stream, -1, -1, 0, 0, false );
			pump.asyncRead( this, null );
			
		}	
		catch(e)
		{
			//logText( "Exception in connect: " + e );
			return e;
		}
		return null;
	},

	/** 
	 * process a message packet.  construct a local object representing the
	 * message, and then pass this to the application via the callback.
	 */
	processMessagePacket: function( packet )
	{
		//logText( "" );
		//logText( "{ Message Packet }" );
		
		// look for headers, pubsub message
		var headersMatch = this.headersElementRE.exec( packet );
		var messageMatch = this.pubsubMessageRE.exec( packet );
		var entry = null;
		var entryString = "";
		var feed = null;
		var sourcefeed = null;
		var title = null;
		var source = null;
		var link = null;
		var sub = "";
		var tmp = null;
		var linkStr = "";
		var lastLinkStr = "";

		// use a pubsub message object
		var msg = new PubsubMessageType();
		
		if( null != messageMatch && null != headersMatch )
		{
			msg.body = messageMatch[0];
			
			// check headers
			do
			{
				entry = this.headerElementRE.exec( headersMatch[0] );
				if( null != entry )
				{
					// logText( "Sub Name: " + this.mapSubscription( entry[1] ) );
					msg.subIDs[msg.subIDs.length] = entry[1];
				}
			}
			while( null != entry );
			
			// find the feed
			
			entry = this.entryElementRE( messageMatch[0] );
			if( null != entry )
			{
				entryString = new String(entry[0]);
				sourcefeed = this.sourceFeedRE( entry[0] );
				if( null != sourcefeed )
				{	
					// get useful stuff here
					source = this.sourceTitleElementRE.exec( sourcefeed[0] );
					if( null != source )
					{
						//logText( "Source: " + source[1] );
						msg.source = source[1];
					}

					linkStr = "";
					lastLinkStr = "";
					do
					{
						link = this.psLinkElementRE.exec( sourcefeed[0] );
						if( null != link )
						{
							tmp = this.getHREFRE.exec( link[0] );
							lastLinkStr = tmp[1];
							if( null != this.checkHTMLType( link[0] )) linkStr = tmp[1];
						}
					}
					while( null != link );
					if( linkStr == "" ) linkStr = lastLinkStr;
					
					msg.sourceLink = linkStr;

					// remove the source-feed from the entry string
					entryString = entryString.replace( this.sourceFeedRE, "" );
				}
				
				msg.title = "Untitled Entry";
				title = this.titleElementRE.exec( entryString );
				if( null != title ) msg.title = scrubCDATA( title[1] );

				msg.date = "(undated)";
				tmp = this.modifiedRE.exec( entryString );
				if( null != tmp ) msg.date = tmp[1];
				
				linkStr = "";
				lastLinkStr = "";
				while( true )
				{
					link = this.linkElementRE.exec( entryString );
					if( null == link ) break;
					tmp = this.getHREFRE.exec( link[0] );
					lastLinkStr = tmp[1];
					if( null != this.checkHTMLType( link[0] )) linkStr = tmp[1];
				}
				if( linkStr == "" ) linkStr = lastLinkStr;
				msg.link = linkStr;

				// ok, now we can call back to the application
				// with this data (assuming there's a callback set)
				
				if( null != this.fnNewMessage ) this.fnNewMessage( msg );
				
			}
		}
		
	},

	/** 
	 * send an affiliations request (gets a list of subscriptions). 
	 */
	sendAffiliations: function( fQuiet )
	{
		var packet = 
			"<iq type='get' from='" + this.username + "' to='" + this.host + "' id='affiliations_verbose'>\n" + 
			"<pubsub xmlns='http://jabber.org/protocol/pubsub'>\n";
		
		if( fQuiet ) packet += "<affiliations/>\n";
		else packet += "<affiliations><ps:verbose xmlns:ps='http://pubsub.com/xmlns'/></affiliations>\n";

		packet += 
			"</pubsub>\n" + 
			"</iq>\n";

		this.outstream.write( packet, packet.length );
	},

	/** 
	 * change our internal connection state 
	 */	
	updateState: function( newState )
	{
		//alert( "ok?" );

		this.connectionState = newState;
		//logText( "Setting state to " + newState );

		//var cb = document.getElementById( "connectButton" );
		//var db = document.getElementById( "disconnectButton" );
		//var obj = null;

		// callback
		if( null != this.fnStateChange ) this.fnStateChange( newState );

		/*
		if( newState == 100 || newState == 0 )
		{
			if( null != cb ) cb.disabled = false;
			if( null != db ) db.disabled = true;
			
			obj = document.getElementById( "username" );
			if( null != obj ) obj.disabled = false;
			obj = document.getElementById( "password" );
			if( null != obj ) obj.disabled = false;
			obj = document.getElementById( "host" );
			if( null != obj ) obj.disabled = false;
		}
		else if( newState > 0 )
		{
			if( null != cb ) cb.disabled = true;
			if( null != db ) db.disabled = false;
			
			obj = document.getElementById( "username" );
			if( null != obj ) obj.disabled = true;
			obj = document.getElementById( "password" );
			if( null != obj ) obj.disabled = true;
			obj = document.getElementById( "host" );
			if( null != obj ) obj.disabled = true;
		}
		*/
	
	},
	
	/** 
	 * callback method when the connection is created 
	 */
	onStartRequest: function(request, context)
	{
		logText( "dataListener.onStartRequest" );

		netscape.security.PrivilegeManager.enablePrivilege("UniversalBrowserRead");
		netscape.security.PrivilegeManager.enablePrivilege("UniversalBrowserWrite");
		netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
		
		logText( "set state to 1" );
		this.updateState( 1 );
		logText( "~ set state to 1" );
		
	},
	
	/** 
	 * callback method when the connection is closed 
	 */
	onStopRequest: function(request, context, status)
	{
		logText( "dataListener.onStopRequest" );
		
		netscape.security.PrivilegeManager.enablePrivilege("UniversalBrowserRead");
		netscape.security.PrivilegeManager.enablePrivilege("UniversalBrowserWrite");
		netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
	
		try
		{
			this.instream.close();
			this.outstream.close();
		}
		catch( e )
		{
			logText( "Exception in close: " + e );
		}

		//logText( "*** connection closed" );			
		//logText( "*** status: " + status );
		
		if( status != 0 )
		{
			// that's probably a connection error - although we
			// might check timing here, to determine if we
			// ever were successful
			
			// on second thought, let's use state to figure that out
			if( this.connectionState == 10 )
			{
				// we were connected (and auth'd) so this is probably not
				// a connection error
				
				// ...
			}
			else
			{
				// could be a conn error...
				//logText( "Probably a connection error" );
				if( null != this.fnError ) this.fnError( 4, "Connection failed." );
				
			}
			
		}
		
		this.updateState( 0 );
		
	},
	
	/** 
	 * callback method when new data arrives (async) 
	 */
	onDataAvailable: function( request, context, inputStream, offset, count )
	{
		netscape.security.PrivilegeManager.enablePrivilege("UniversalBrowserRead");
		netscape.security.PrivilegeManager.enablePrivilege("UniversalBrowserWrite");
		netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");

		try
		{
			// what choice do we have? take out newlines
			var strData = this.instream.read(count);
			strData = strData.replace( /(?:\n|\r)/g, " " );
			this.data += strData;
			
			// process the stream.  
			this.processDataStream();
		}
		catch( e )
		{
			this.CloseConnection();
			//logText( "Exception in onDataAvailable: " + e );
			//logText( "Calling close" );
		}
		
	},
	
};	

//////////////////////

/**
 * add an event listener on load to call the autoconnect.  we need
 * to have the window built before this point, otherwise we could
 * do it now.
 */
window.addEventListener( "load", doAutoConnect, false );

/**
 * and start the service check timer...
 */
setTimeout( "serviceCheck()", SERVICE_CHECK_TIMEOUT * 1000 );

