/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the PubSub Sidebar.
 *
 * The Initial Developer of the Original Code is
 * PubSub Concepts, Inc.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
 
 var iSpanIndex = 0;

// this function changes the style of the span tags added
// to the document to turn highlighting on and off.  this 
// should recurse through documents and frames too.  the 
// open indexes are used so it works for cached pages.

function pubsubToggleHighlightsOn(doc, vOn)
{
    var i, obj, id;
    if( vOn )
    {
       for( i = 0; i< 1000; i++)
       {
	  id = "pshid_" + i;
	  obj = doc.getElementById( id );
	  if( obj == null) break;
	  obj.style.color="#000000";
	  obj.style.backgroundColor="#ffff66";
	  obj.style.fontWeight="bold";
	}
    }
    else
    {
       for( i = 0; i< 1000; i++)
       {
	   id = "pshid_" + i;
	   obj = doc.getElementById( id );
	   if( obj == null) break;
	   obj.style.color="";
	   obj.style.backgroundColor="";
	   obj.style.fontWeight="";
	}
    }
}

function pubsubHighlightWindow(oWin, on)
{
   if(oWin.document != null &&
      oWin.document != undefined)
   {
      if(oWin.document.body != null &&
         oWin.document.body != undefined)
      {
        pubsubToggleHighlightsOn(oWin.document, on);
      }
      else if(oWin.document.childNodes != null &&
              oWin.document.childNodes != undefined)
      {
        for(var i = oWin.document.firstChild; i != null; i = i.nextSibling)
        {
          pubsubToggleHighlightsOn(oWin.document, on);
        }
      }
   }
   else if(oWin.frames != null &&
           oWin.frames != undefined && 
           oWin.frames.length > 0)
   {
      for(var i = 0; i < oWin.frames.length; i++)
      {
         if(oWin.frames[i] != null &&
            oWin.frames[i] != undefined)
         {
           pubsubHighlightWindow(oWin.frames[i], on);
         }
      }
   }

}

// Javascript by default sort data in lexicon order.
// By supplying length of string comparing function to sort by value

function pubsubCompareStringLength(a, b)
{
   return (a.length - b.length);
}

// Javascript by default sort data in lexicon order.
// By supplying number comparing function to sort by value

function pubsubCompareNumber(a, b)
{
  return (a - b);
};

// Split the data strings in to array of words,
// Marked matching words and missed words seperately
// Create tag element and set highlight fgcolor and bgcolor
// Replace current node with newly created text nodes and 
// span nodes.


function pubsubAddHighlightSpanTags( oDoc, oParent, oNode, 
                                     arrStartFinal, arrEndFinal )
{
   arrStartFinal.sort(pubsubCompareNumber);
   arrEndFinal.sort(pubsubCompareNumber); 
   
   var data = oNode.data;
   var totalSegments = arrStartFinal.length;

   for(var i = 0; i < arrStartFinal.length; i++)
   {
      if(i == 0)
      {
        if(arrStartFinal[i] > 0)
        {
          totalSegments++;
        }
      }

      if(i == (arrStartFinal.length-1))
      {
         if(arrEndFinal[i] < (data.length-1))
         {
           totalSegments++;
         }
      }

      if(i < (arrStartFinal.length-1))
      {
         if(arrEndFinal[i] < arrStartFinal[i+1])
         {
           totalSegments++;
         }
      }
    }   

    var arrSegmentText = new Array(totalSegments);
    var arrSegmentTag = new Array(totalSegments);

    var iStep = 0;
    for(var i = 0; i < arrStartFinal.length; i++)
    {
        if(i == 0)
        {
           if(arrStartFinal[i] > 0)
	   {
	      arrSegmentText[iStep] = data.substring(0, arrStartFinal[i]);
              arrSegmentTag[iStep++] = 0;
           }
        }

        arrSegmentText[iStep] = data.substring(arrStartFinal[i], arrEndFinal[i]+1);
        arrSegmentTag[iStep++] = 1;

        if(i < (arrStartFinal.length - 1))
	{
           arrSegmentText[iStep] = data.substring(arrEndFinal[i]+1, arrStartFinal[i+1]);
           arrSegmentTag[iStep++] = 0;
        }

        if(i == (arrStartFinal.length - 1))
	{
	  if(arrEndFinal[i] < (data.length-1))
	   {
	       arrSegmentText[iStep] = data.substring(arrEndFinal[i]+1, data.length);
	       arrSegmentTag[iStep] = 0;
           }
        }
    }

    var arrSegmentNodes = new Array(totalSegments);

    for(var i = 0; i < arrSegmentText.length; i++)
    {
      if(arrSegmentTag[i] == 0)
      {
         arrSegmentNodes[i] = oDoc.createTextNode(arrSegmentText[i]);
      }
      else
      {
        // Mozilla version, Mozilla support standard DOM 
        // method

        var spanChildNode = oDoc.createTextNode(arrSegmentText[i]);
        arrSegmentNodes[i] = oDoc.createElement("span");
        arrSegmentNodes[i].setAttribute("id", "pshid_" + iSpanIndex);
        arrSegmentNodes[i].appendChild(spanChildNode);
	iSpanIndex++;
      }
    }

    var oNextNode = oNode.nextSibling;

    if(oNode.nextSibling == null ||
       oNode.nextSibling == undefined)
    {
      oParent.removeChild(oNode);
      for(var i = 0; i < totalSegments; i++)
      {
        oParent.appendChild(arrSegmentNodes[i]);
      }
    }
    else
    {
      oParent.removeChild(oNode);
      for(var i = 0; i < totalSegments; i++)
      {
        oParent.insertBefore(arrSegmentNodes[i], oNextNode);
      }
    }  
}


// Give a node, match the text node data to see
// whether keywords exists.
// The algorithm will match substring
// if matches exists, passes to span maker to create
// span tag


function pubsubFindHighlightPhrasesInText(oDoc, oNode, arrKeywords){
   
   var parent = oNode.parentNode;

   if(oNode.data == null ||
      oNode.data == undefined ||
      oNode.data.length ==  0)
   {
     return;
   }
     
   var currentData = new String(oNode.data.toLowerCase());
   var currentTag = new Array(oNode.data.length);

   for(var i = 0; i < currentTag.length; i++)
   {
     currentTag[i] = -1;
   }

   var arrSplitStartIndices = new Array(currentData.length);
   var arrSplitEndIndices = new Array(currentData.length);
   var iSplitIndicesLast = 0;

   for(var i = 0; i < arrKeywords.length; i++)
   {
      var currentKeyword = new String(arrKeywords[i].toLowerCase());
      var currentIndex = currentData.indexOf(currentKeyword);

      while(currentIndex > -1)
      {
         for(var j = currentIndex; 
             j < (currentIndex + currentKeyword.length);
             j++)
         {
            if(currentTag[j] == -1)
            {
              currentTag[j] = i;
	      if(j == currentIndex)
              {
                arrSplitStartIndices[iSplitIndicesLast] = j;
              }
              if(j == (currentIndex + currentKeyword.length - 1))
              {
                arrSplitEndIndices[iSplitIndicesLast++] = j;
              }
            }           
         } 
         currentIndex = currentData.indexOf(currentKeyword,
                                            currentIndex + currentKeyword.length);
      }     
   }
   // Keyword exists

   if(iSplitIndicesLast > 0)
   {
      var arrStartFinal = new Array(iSplitIndicesLast);
      var arrEndFinal = new Array(iSplitIndicesLast);
      for(var i = 0; i < iSplitIndicesLast; i++)
      {
        arrStartFinal[i] = arrSplitStartIndices[i];
        arrEndFinal[i] = arrSplitEndIndices[i];
      }
      
      pubsubAddHighlightSpanTags(oDoc, parent, oNode, arrStartFinal, arrEndFinal);    
   } 
}


// Recursively traverse documents
// We only interested in text leaf nodes.
// Handle CDATA later if CDATA is not text node (need to check DOM book again)


function pubsubTraverseDoc(oDoc, oNode, arrKeywords)
{
   if(oNode == null ||
      oNode == undefined)
   {
     return;
   }
   else if(oNode.nodeType == 3)
   { 
      pubsubFindHighlightPhrasesInText(oDoc, oNode, arrKeywords);
   }
   else 
   { 
      if(oNode.childNodes != null)
      {
        for(var i = oNode.firstChild; i != null; i = i.nextSibling)
        {
           pubsubTraverseDoc(oDoc, i, arrKeywords);
        }
      }
   } 
}


// Tokenize Highlight keyword string into array of keywords
// May change in future
// 


function pubsubTokenizeQueryString(strKeywords)
{
   var arrPhrases;
   var arrWords;
  
   reFindPhrases = /\"[^\n\"]+\"/ig; 
   reQuotes = /\"/g;

   arrPhrases = strKeywords.match(reFindPhrases);

   if(arrPhrases != null &&
      arrPhrases != undefined)
   {
     for(var i = 0; i < arrPhrases.length; i++)
     { 
        arrPhrases[i] = arrPhrases[i].replace(reQuotes, "");  
     }
     strKeywords = strKeywords.replace(reFindPhrases, "");
   }

   if(strKeywords.length > 0)
   {
     reSpaces = /\s+/;
     strKeywords = strKeywords.replace(reSpaces, " ");
     reTrimSpace = /^\s+|\s+$/;
     strKeywords = strKeywords.replace(reTrimSpace, "");
     arrWords = strKeywords.split(reSpaces);
   }
   
   var result;

   if(arrPhrases != null &&
      arrPhrases != undefined &&
      arrWords != null &&
      arrWords != undefined)
   { 
     
      result = arrPhrases.concat(arrWords);
   }
   else if(arrPhrases != null &&
           arrPhrases != undefined &&
           arrWords  == null &&
           arrWords == undefined)
   {
      result = arrPhrases;
   }
   else if(arrPhrases == null &&
           arrPhrases == undefined &&
           arrWords != null &&
           arrWords != undefined)
   {
      result = arrWords;
   }
   
   if(result != null &&
      result != undefined)
   {
     result.sort(pubsubCompareStringLength);
     result.reverse();
   }

   return result;
}

// Generate Regular Expression of keywords

function pubsubCreateKeywordRegExps(arrKeywords)
{
   var reResult;
   if(arrKeywords != null &&
      arrKeywords != undefined)
   {
      reResult = new Array(arrKeywords.length);

      for(var i = 0; i < arrKeywords.length; i++)
      {
        reResult[i] = new RegExp(arrKeywords[i], "ig");
      }
   }

   return reResult;
}

// Traverse windows to find documents

function pubsubTraverseWindow(oWin, arrKeywords, regKeywords)
{
   if(oWin.document != null &&
      oWin.document != undefined)
   {
      if(oWin.document.body != null &&
         oWin.document.body != undefined)
      {
	iSpanIndex = 0;
        pubsubTraverseDoc(oWin.document, oWin.document.body,  arrKeywords);
      }
      else if(oWin.document.childNodes != null &&
              oWin.document.childNodes != undefined)
      {
        for(var i = oWin.document.firstChild; i != null; i = i.nextSibling)
        {
	  iSpanIndex = 0;
          pubsubTraverseDoc(oWin.document, i, arrKeywords);
        }
      }
   }
   else if(oWin.frames != null &&
           oWin.frames != undefined && 
           oWin.frames.length > 0)
   {
      for(var i = 0; i < oWin.frames.length; i++)
      {
         if(oWin.frames[i] != null &&
            oWin.frames[i] != undefined)
         {
           pubsubTraverseWindow(oWin.frames[i], arrKeywords, regKeywords);
         }
      }
   }
}

// remove nulls, duplicates

function pubsubFlattenArray(arr)
{
    var rslt = new Array();
    var i, j, ilen = arr.length;
    var iIndex = 0;	

    for( i = 0; i< ilen; i++)
    {
       if( arr[i].length > 0 )
       {
	   for( j = 0; j< i; j++)
	   {
	       if( arr[i] == arr[j] ) break;
	   }
	   if( j == i ) rslt[iIndex++] = arr[i];
	}
     }
	
    return rslt;
}

// Create Highlights for PubSub Sidebar

function pubsubCreateHighlights(oWin, strKeywords)
{
   if(oWin == null ||
      oWin == undefined)
   {
      return;
   }

   var arrTmp = pubsubTokenizeQueryString(strKeywords);
   var arrKeywords = pubsubFlattenArray(arrTmp);

   var regKeywords = pubsubCreateKeywordRegExps(arrKeywords);

   if(arrKeywords != null && 
      arrKeywords != undefined &&
      regKeywords.length > 0)
   {
     pubsubTraverseWindow(oWin, arrKeywords, regKeywords);
   }
}

/*
function pubsubCheckLoad()
{
	alert( "Load OK" );
}
*/
