/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the PubSub Sidebar.
 *
 * The Initial Developer of the Original Code is
 * PubSub Concepts, Inc.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
 
//========================================================
//
// the sidebar.  
//
// jabber connection code runs independently under the 
// window so that it can stay connected when the sidebar 
// is closed.
// 
//========================================================

/**
 * the maximum length of subscription names before they're
 * truncated.  this is for the dropdown list, so it doesn't
 * force the page to be too wide.
 */
const MAX_SUBNAME_LENGTH = 28;

/**
 * the maximum length of a single word before we break it - 
 * for display wrapping.  see above also.
 */
const MAX_WORD_LENGTH = 35;

/** 
 * message div tags 
 */
var MessageDivSet = null;

/**
 * the current displayed bucket.
 * "" (empty string) means "all subs".
 */
var CurrentBucket = "";

/**
 * the "wait" cursor is set from time
 * to time around async operations, so we need
 * to know who's using it.
 */
var WaitCursorRefCount = 0;

/** 
 * the "display" window - holds message divs 
 */
var DisplayDocument = null;

/** 
 * the "controls" window 
 */
var ControlsDocument = null;

/** 
 * the preferences window 
 */
var OptionsDocument = null;

/** 
 * the log window, for debug
 */
var LogDocument = null;

/** 
 * keywords for highlighting 
 */
var lastKeywords = "";

/** 
 * have we already highlighted the document? note
 * that this starts true so it'll never get called
 * until we want it...
 */
var fHighlightsSet = true;

/** 
 * the last window we highlighted 
 */
var wndHighlights = null;

/** 
 * instance of app prefs access object 
 */
var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

/**
 * reference to the connection "server" 
 */
var ConnectionServer = null;

/**
 * highlights on or off
 */
var fHighlightState = true;

/**
 * show close boxes
 */
var fCloseBoxes = true;

/** 
 * show dates 
 */
var fShowDates = true;

/** 
 * extended data - source, date, [subs]
 */
var fShowExtended = true;

/** 
 * load history for subscriptions (on first view)
 */
var fLoadHistory = true;

/** 
 * close on link.  this is an option.
 */
var fCloseOnLink = false;

/**
 * we can override the close on link - we do that on some links
 * (on the help button, for example)
 */
var fOverrideClose = false;

/**
 * display help.  for now this opens a web page, but that may change...
 */
function showHelpContent()
{
	// if close on link is set, don't obey it
	fOverrideClose = true;
	var wnd = window.parent.document.getElementById( "content" );
	if( null != wnd )
	{
		wnd.contentWindow.location.href="http://www.pubsub.com/sidebar-firefox";
	}
}

/** 
 * log fn 
 */
function doClearLog()
{
	if( null == LogDocument ) return;
	var obj = LogDocument.getElementById("outputArea");
	if( null != obj ) obj.innerHTML = "";
}

/** 
 * log fn 
 */
function logText( str )
{
	if( null == LogDocument )
	{
		return;
	}
	var obj = LogDocument.getElementById("outputArea");
	if( null != obj )
	{
		str = str.replace( /\</g, "&lt;" );
		str = str.replace( /\>/g, "&gt;" );
		str = str.replace( /\&/g, "&amp;" );
		
		obj.innerHTML += str;
		obj.innerHTML += "<BR/>\n";		

		obj = LogDocument.getElementById("bottomOfLog");
		if( null != obj ) obj.scrollIntoView();
	}
}

/**
 * show the options screen, allows login and so on
 */
function showOptions()
{
	/*
	var ifr = document.getElementById( "ifrDisplay" );
	if( null != ifr )
	{
		ifr.contentWindow.location.href='chrome://pubsubsidebar/skin/options.html';
	}
	*/
	window.parent.fAllowOpen = true;
	window.location.href = "chrome://pubsubsidebar/content/sidebar-options.xul";
}

/**
 * back to the normal screen (from options)
 */
function showSidebarDisplay()
{
	window.parent.fAllowOpen = true;
	window.location.href = "chrome://pubsubsidebar/content/sidebar.xul";
}

/** 
 * check reference
 */
function ensureServerConnection()
{
	if( null == ConnectionServer )
	{
		ConnectionServer = window.parent;
		if( null != ConnectionServer )
		{
			ConnectionServer.registerClient( window );
			logText( "Registered" );
		}
	}
}

/**
 * reset display variables 
 */
function resetAll()
{
	CurrentBucket = "";
	buildDivSet();
	resetAllDivs();

	var obj = FindSubsList();
	if( null != obj )
	{
		// got to get stuff out...
		do{ tmp = obj.removeItemAt( 0 ); }
		while( null != tmp );
	}

}

/**
 *
 */
function FindSubsList()
{
	return ControlsDocument.getElementById( "subsList" );
}


/**
 * look at the page, get the placeholder div and then
 * look for all divs in there (with the recursive function).
 * this builds an ordered set of display units.
 */
function buildDivSet( )
{
	logText( "Building div set" );
	var msgArea = DisplayDocument.getElementById( "messagesArea" );
	MessageDivSet = new Array();
	if( null != msgArea ) DivSetFn( msgArea );
}

/**
 * this is the recursive function for finding div tags
 * with the appropriate class in the page.
 */
function DivSetFn( par )
{	
	var i = 0;
	var cn = par.childNodes;
	
	if( null != cn )
	{
		for( i = 0; i< cn.length; i++)
		{
			if( cn[i].nodeName == "DIV" )
			{
				if( cn[i].className == "msg" )
				{
					// push reference to this div onto our stack
					MessageDivSet[MessageDivSet.length] = cn[i];
				}
				else DivSetFn( cn );
			}
			else DivSetFn( cn );
		}
	}
}

/**
 * reset message unit divs on the page - render them empty.
 */
function resetAllDivs()
{
	var i = 0;
	if( null == MessageDivSet ) buildDivSet();
	if( MessageDivSet.length > 0 )
	{
		for( i = 0; i< MessageDivSet.length; i++ )
		{
			MessageDivSet[i].innerHTML = "";
		}
	}	
}

function onLogText( str )
{
	logText( str );
}

function stopConnection()
{
	setDocuments();
	ensureServerConnection();
	
	if( ConnectionServer.isConnected())
	{
		ConnectionServer.doCloseConnection();
	}
	else
	{
		logText( "Server is not connected" );
	}
}

/**
 * command handler: open server connection
 */
function startConnection()
{
	setDocuments();
	
	logText( "Ensure server" );
	
	ensureServerConnection();
	
	if( ConnectionServer.isConnected())
	{
		logText( "Already connected" );
		loadAndConnect();
		return;
	}
	
	logText( "calling resetAll()" );

	resetAll();
	
	logText( "calling get parameters" );
	
	var parameters = getConnectionParameters();
	
	logText( "calling doConnect(): " + ConnectionServer );
	
	ConnectionServer.doConnect( parameters );
	
}

/**
 * ensure we have pointers to our documents
 */
function setDocuments()
{
	try
	{
		var o = null;
		if( null == DisplayDocument ) 
		{
			o = document.getElementById( "ifrDisplay" );
			if( null != o ) DisplayDocument = o.contentDocument;
	
			//o = document.getElementById( "ifrControls" );
			//if( null != o ) ControlsDocument = o.contentDocument;
			ControlsDocument = document;
	
			//o = document.getElementById( "ifrOptions" );
			//if( null != o ) OptionsDocument = o.contentDocument;
			var testObj = document.getElementById( "options_main" );
			if( null != testObj ) OptionsDocument = document;
	
			o = document.getElementById( "ifrLog" );
			if( null != o ) LogDocument = o.contentDocument;
		}
	}
	catch( e ){ alert( "E: " + e ); }

}

///////////////

/**
 * remove this message.
 */
function deleteID( msgID )
{
	// alert( "Delete: " + msgID );
	
	// tell the parent to delete the message
	ConnectionServer.deleteID( msgID, CurrentBucket );
	
	// now repaint the screen...
	logText( "Repainting..." );
	var subID = CurrentBucket;
			
	// set to blank so it'll force a repaint
	CurrentBucket = "{none}";
	doDisplaySub( subID );
}

/**
 * render a message as html.  messages should only be rendered
 * once (again if the selected subscription changes) and then 
 * pushed down as html.  this function paints an additional field
 * if we're displaying "recent items" - the list of matched 
 * subscriptions.
 */
function formatMessageContents( msg )
{
	var i = 0;
	var strHTML = "";
	var kwString = "";
	var kw = "";
	
	for( i = 0; i< msg.subIDs.length; i++)
	{
		var subObj = ConnectionServer.SubscriptionMap[msg.subIDs[i]];
		if( null != subObj )
		{
			// remove commas, we use them as separators
			kw = subObj.keywords.replace( /,/g, "" );
			kwString += (subObj.keywords + " ");
		}
	}
	
	// now sub new commas for the old quotes
	kwString = kwString.replace( /"/g, "," );
	kwString = kwString.replace( /^,/, "" );
	kwString = kwString.replace( /,$/, "" );
	kwString = kwString.replace( /,\s*,/g, "," );
	kwString = kwString.replace( /,\s*,/g, "," );
	
	// close box: optional
	if( fCloseBoxes )
	{
		strHTML += "<div style='border-top: 1px solid #a0a0a0; text-align: right; font-size: smaller; '>" +
				   "<a style='cursor: hand; cursor: pointer;' onclick='window.parent.deleteID(" + msg.messageID + ");'>X</a></div>";
	}
	
	// title (and link)
	if( msg.link != "" ) 
	{
		strHTML += "<div class='headline'><a onclick='window.parent.setTargetKeywords(\"" + kwString + "\", \"" + msg.link + "\" );' target='_content' href='" + msg.link + "'>" + addWordBreaks(scrubTagsOut(msg.title)) + "</a></div>\n";
	}
	else strHTML += "<div class='headline'>" + addWordBreaks(msg.title) + "</div>\n";
	
	// source, date: optional
	if( fShowExtended )
	{
		if( fShowDates )
		{
			if( msg.sourceLink != "" ) 
				strHTML += "<div class='source'><a onclick='window.parent.setTargetKeywords(\"bush,kerry\", \"" + msg.sourceLink + "\" );' target='_content' href='" + msg.sourceLink + "'>" + addWordBreaks(msg.source) + "</a></div>\n";
			else strHTML += "<div class='source'>" + addWordBreaks(msg.source) + "</div>\n";
			strHTML += "<div class='date'>" + msg.date + "</div>\n";
		}
		else
		{
			if( msg.sourceLink != "" ) 
				strHTML += "<div class='sourceonly'><a onclick='window.parent.setTargetKeywords(\"bush,kerry\", \"" + msg.sourceLink + "\" );' target='_content' href='" + msg.sourceLink + "'>" + addWordBreaks(msg.source) + "</a></div>\n";
			else strHTML += "<div class='sourceonly'>" + addWordBreaks(msg.source) + "</div>\n";
		}

		// add the list of subscriptions, if we're not displaying a particular (selected) one
		if( "" == CurrentBucket 
			|| null == CurrentBucket )
		{
			strHTML += "<div class='subscriptions'>";
			for( i = 0; i< msg.subIDs.length; i++)
			{
				if( i > 0 ) strHTML += ", ";
				if( null != ConnectionServer.SubscriptionMap[msg.subIDs[i]]) strHTML += ConnectionServer.SubscriptionMap[msg.subIDs[i]].name;
				else strHTML += msg.subIDs[i];
			}
			strHTML += "</div>";
		}
	}
	
	return strHTML;	
}

/**
 * callback method: when a new "affiliations" response is
 * received, we dump old subs and rebuild the list.  this
 * prevents doubling up.
 */
function onResetSubs()
{
	logText( "Resetting subs" );

	var i = 0;
	var len = 0;
	var obj = FindSubsList();
	var tmp = null;

	try
	{
		if( null != obj )
		{
			// enable this, too: it starts gray
			obj.disabled = false; 

			// got to get stuff out...
			do{ tmp = obj.removeItemAt( 0 ); }
			while( null != tmp );
			
			obj.appendItem( "Latest Messages", "" ); 
		}
	}
	catch(e)
	{
		// alert("E [122]: " + e);
	}
}

function onSubListComplete()
{

	try
	{
		var subsListObj = FindSubsList();
		if( null != subsListObj && subsListObj.firstChild.childNodes.length > 0 )
		{
			// ??
			//subsListObj.selectedIndex = 1;
			//subsListObj.value = subsListObj.selectedItem.label;
			
			var items = subsListObj.firstChild.childNodes;
			subsListObj.selectedItem = items[1];

			//alert( "selitem: " + subsListObj.selectedItem );

			// force...
			CurrentBucket = "{null}";
			doUpdateSub();
		}
	}
	catch(e)
	{
		// alert("E [123]: " + e);
	}
}

/** 
 * callback method: will be called on new message arrival 
 */
function onNewMessage( msg )
{
	var i = 0;
	var fDisplay = false;
	
	// check if we need to display this one
	for( i = 0; i< msg.subIDs.length; i++ )
	{
		if( msg.subIDs[i] == CurrentBucket ) fDisplay = true;
	}

	// update the screen, if necessary
	if( fDisplay 
		|| "" == CurrentBucket 
		|| null == CurrentBucket )
	{
		// build the page div set, if necessary
		if( null == MessageDivSet ) buildDivSet();

		// and push onto the screen
		if( MessageDivSet.length > 0 )
		{
			for( i = MessageDivSet.length - 1; i > 0; i--)
			{
				MessageDivSet[i].innerHTML = MessageDivSet[i-1].innerHTML;
				//MessageDivSet[i].style.display = "inline";
			}
			MessageDivSet[0].innerHTML = formatMessageContents( msg );
		}	
	}
}


/** 
 * app function: get connection parameters.  use prefs!
 */
function getConnectionParameters()
{
	var o = null;
	var d = new Date();
	var parameters = new Array();

	if( null == prefs )
		prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
	var prefsSet = prefs.getBoolPref( "com.pubsub.pubsubsidebar.prefs_set" );
	
	if( prefsSet )
	{
		var tmp = "";
		var fTmp = false;
		
		parameters["username"] = prefs.getCharPref( "com.pubsub.pubsubsidebar.username" );
		parameters["password"] = prefs.getCharPref( "com.pubsub.pubsubsidebar.password" );
		parameters["host"] = prefs.getCharPref( "com.pubsub.pubsubsidebar.host" );
		
	}
	else
	{
		// set defaults
		parameters["username"] = "sample@pubsub.com";
		parameters["password"] = "sample";
		parameters["host"] = "xmpp.pubsub.com";
	}
	parameters["resource"] = "firefox-" + d.getMilliseconds();
	
	return parameters;
}


/**
 * add breaks in long words 
 */
function addBreaks( str )
{
	var strOut = "";
	while( str.length >  MAX_WORD_LENGTH )
	{
		strOut += str.substring(0, MAX_WORD_LENGTH);
		strOut += " ";
		str = str.substring( MAX_WORD_LENGTH );
	}
	strOut += str;
	return strOut;
}

/**
 * get tags out of the string for display - since we're
 * not permitting links...
 */
function scrubTagsOut( str )
{
	var strOut = str;
	strOut = strOut.replace( /\&lt;.+?\&gt;/, "" );
	strOut = strOut.replace( /\<.+?\>/, "" );
	return strOut;
}

/**
 * break long words, helps display 
 */
function addWordBreaks( str )
{
	var strOut = "";
	var arr = str.split( /\s+/ );
	var i = 0;
	
	for( i = 0; i< arr.length; i++)
	{
		if( i > 0 ) strOut += " ";
		if( arr[i].length > MAX_WORD_LENGTH ) strOut += addBreaks( arr[i] );
		else strOut += arr[i];
	}
	
	return strOut;
}


/** 
 * callback method: will be called on error.  
 */
function onJabberError( errorCode, errorString )
{
	//var obj = DisplayDocument.getElementById( "error" );
	var obj = ControlsDocument.getElementById( "error" );
	if( null != obj )
	{
		// clear?
		if( errorCode == 0 || null == errorString ) obj.innerHTML = "";
		else obj.innerHTML = 
			"ERROR [" + errorCode + "]: " + errorString + "<BR/><BR/>";
	}
	else
	{
		if( errorCode != 0 && null != errorString ) 
			alert( "ERR [" + errorCode + "]: " + errorString );
	}
}

/**
 * truncate subscription names to [X] characters, 
 * so the dropdown doesn't become too wide
 */
function shortenSubName( str )
{
	if( str.length <= MAX_SUBNAME_LENGTH ) return str;
	str = str.substring(0, MAX_SUBNAME_LENGTH);
	str += "...";
	return str;
}


/** 
 * callback method: will be called on new subscription, so
 * we need to add it to the dropdown list.
 */
function onNewSub( sub )
{
	logText( "OnNewSub" );

	try
	{
		var opt = null;
		var obj = FindSubsList();
		
		// has to have a legit name
		if( sub.name == "" )
		{
			logText( "ERR: invalid sub name for ID " + sub.ID );
			return;
		}
		
		if( null != obj )
		{
			//opt = DisplayDocument.createElement("OPTION");
			//opt.text = shortenSubName( sub.name );
			//opt.value = sub.ID;
			//obj.options.add(opt);
	
			obj.appendItem( sub.name , sub.ID );
	
			// map local
			// server // SubscriptionMap[sub.ID] = sub;
		}
	}
	catch(e)
	{
		//alert("E [124]: " + e);
	}

}

/////////////////////////

function onSubHistoryComplete( subID )
{
	logText( "SHC" );
	if( subID == CurrentBucket )
	{
		// repaint the sub 
		logText( "Repainting..." );
			
		// set to blank so it'll force a repaint
		CurrentBucket = "";
		doDisplaySub( subID );
	}
}

/** callback */
function onError( code, str )
{
	if( code == 0 )
	{
		// clear
		if( null != DisplayDocument )
		{
			var obj = DisplayDocument.getElementById("error");
			if( null != obj )
			{
				obj.innerHTML = "";
				obj.style.display = "none";
			}
		}
		
	}
	else
	{
		logText( "ERR " + code + ": " + str );
		if( null != DisplayDocument )
		{
			var obj = DisplayDocument.getElementById("error");
			if( null != obj )
			{
				var errHTML = "<div class='errorheader'>Error " + code + ":</div>\n" +
							  "<div class='errorcontent'>" + str + "</div>";
				obj.innerHTML = errHTML;
				obj.style.display = "inline";
			}
		}
	}
}

/**
 * set keywords for the next highlight operation.
 * this is useful for the "new" set, which can
 * have keywords from multiple subscriptions.
 */
function setTargetKeywords( strKW, strTarget )
{
	var i = 0;
	var kwArr = strKW.split( /,/ );
	
	lastKeywords = "";
	for( i = 0; i< kwArr.length; i++)
	{
		kwArr[i] = kwArr[i].replace( /^\s+/, "" );
		if( kwArr[i].length > 0 )
		{
			lastKeywords += ( "\"" + kwArr[i] + "\" " );
		}
	}
	//alert( "STK: " + lastKeywords );

	fHighlightsSet = false;
	strHighlightTarget = strTarget;
}

/**
 * display state info
 */
function setStateInfo()
{
	var stateInfo = " ";
	var obj = ControlsDocument.getElementById( "info" );
	
	// are we connected?
	ensureServerConnection();
	if( ConnectionServer.isConnected() )
	{
		try
		{
			//var parms = ConnectionServer.getAccountPrefs();
			//stateInfo = "connected: " + parms["username"];
			stateInfo = "connected: " + ConnectionServer.ConnectedUsername;
		}
		catch( e )
		{
			alert( "E: " + e );
		}
	}
	else
	{
		stateInfo = "not connected";
	}
	
	//if( null != obj ) obj.innerHTML = stateInfo;
	if( null != obj ) obj.value = stateInfo;
}

/**
 * show/hide the various highlight buttons 
 */
function setHighlightSpanState()
{
	var o = null;
	o = ControlsDocument.getElementById( "showHighlightsOn" );
	if( null != o ) o.style.display = ( fHighlightState ? "none" : "inline" );
	o = ControlsDocument.getElementById( "showHighlightsOff" );
	if( null != o ) o.style.display = ( fHighlightState ? "inline" : "none" );
}

/**
 * toggle highlights, depending on the state of the checkbox 
 */
function doShowHighlights( fState )
{
	// var st = getHighlightButtonState();
	fHighlightState = fState;
	if( null != wndHighlights ) pubsubHighlightWindow( wndHighlights, fState );
	setHighlightSpanState();
}

/**
 * fetch the checkbox state
 */
function getHighlightButtonState()
{
	//var o = document.getElementById( "highlights" );
	//if( null != o ) return o.checked;
	//return false;
	return fHighlightState;
}

/** 
 * callback method called when the document is complete 
 * (actually, when the stop button goes dark).
 */
function loadEventFired( wnd )
{
	//logText( "wnd: " + wnd );
	//logText( "wnd.location: " + wnd.location );
	// logText( "wnd.status: " + wnd.status );
	
	if( !fHighlightsSet )
	{
		try
		{
			logText( "Calling create highlights..." );
			pubsubCreateHighlights( wnd, lastKeywords );
			
			// get state of highlight button
			if( getHighlightButtonState()) 
			{
				logText( "Calling highlight..." );
				pubsubHighlightWindow( wnd, true );
			}
			else logText( "NOT HIGHLIGHTING" );
			
			// success, set flag
			fHighlightsSet = true;
			
			// hang on to the window for toggling
			wndHighlights = wnd;
		}
		catch( e )
		{
			logText( "E: " + e );
		}
	}
	
	// close?
	if( fCloseOnLink && !fOverrideClose) 
		window.parent.toggleSidebar('viewPubSubSidebar');		
	
	// fix override
	if( fOverrideClose ) fOverrideClose = false;
	
}

/** 
 * manage the wait/auto cursor states 
 */
function refWaitCursor( count )
{
	var oldCount = WaitCursorRefCount;
	WaitCursorRefCount += count;
	if( WaitCursorRefCount < 0 ) WaitCursorRefCount = 0;

	if( WaitCursorRefCount == 0 && oldCount != 0 )
	{
		//window.setCursor( "auto" );
		if( null != DisplayDocument )
		{
			DisplayDocument.body.style.cursor = "auto";
		}
	}
	else if( WaitCursorRefCount != 0 && oldCount == 0 )
	{
		//window.setCursor( "wait" );
		//alert( "DisplayDocument: " + DisplayDocument );
		//alert( "DisplayDocument.body: " + DisplayDocument.body );
		
		DisplayDocument.body.style.cursor = "wait";
	}
}

/**
 * select a sub from the list and render that sub to the screen
 */
function doUpdateSub()
{
	var obj = FindSubsList();
	if( null != obj )
	{
		doDisplaySub( obj.value );
	}
}

/**
 * toggle the visible button, depending on the subscription
 * selected.
 */
function updateViewAsRSSButton()
{
	var canShow = true;
	var o = null;

	if( null == CurrentBucket || CurrentBucket == "" ) canShow = false;
	
	o = ControlsDocument.getElementById( "showRSSButtonHidden" );
	if( null != o ) o.style.display = ( canShow ? "none" : "inline" );
	
	o = ControlsDocument.getElementById( "showRSSButtonVisible" );
	if( null != o ) o.style.display = ( canShow ? "inline" : "none" );

}

/**
 * display the current selection as an RSS file in the 
 * browser window
 */
function displayAsRSS()
{
	var obj = FindSubsList();
	var wnd = null;
	var subID = "";
	var RSSLink = "";
	
	if( null != obj )
	{
		subID = obj.value;
		if( null != subID 
			&& subID != "" )
		{
			// construct the rss link...
			subID = subID.replace( /^pubsub\/subs\//, "" );
			RSSLink = "http://rss.pubsub.com/" + subID + ".xml";
		
			wnd = window.parent.document.getElementById( "content" );
			if( null != wnd )
			{
				wnd.contentWindow.location.href = RSSLink;
			}
		}
	}
}

/**
 * app function: render the passed sub to the screen
 * and set as current for display purposes
 */
function doDisplaySub( subID )
{
	var i = 0;
	var max = 0;
	var arr = null;

	// normalize
	if( null == subID ) subID = "";

	// check if we're already displaying this one - if so, do nothing
	if( subID == CurrentBucket ) return;
	
	// logText( "switching to sub: " + subID );
	
	// set as current
	CurrentBucket = subID;

	// one new thing - set the appropriate option for
	// "view as rss" - if it's the latest/all items feed,
	// then that's not available.
	updateViewAsRSSButton();

	// get the bucket
	arr = ConnectionServer.SubscriptionBuckets[ subID ];

	// store this...
	ConnectionServer.LastSelectedSub = subID;

	// ok // THIS NEEDS TO MOVE TO SEVER...
	if( fLoadHistory ) ConnectionServer.ensureSubHistory( subID );

	// clear the screen
	if( null == MessageDivSet ) buildDivSet();
	resetAllDivs();

	// may be no need to paint
	if( null == arr || arr.length == 0 ) return;

	// now paint - in this case, we can paint forward rather than backwards
	if( MessageDivSet.length > 0 && arr.length )
	{
		max = MessageDivSet.length;
		if( max > arr.length ) max = arr.length;
		
		// logText( "painting " + max + " messages" );
		
		for( i = 0; i< max; i++ )
		{
			MessageDivSet[i].innerHTML = formatMessageContents( arr[i] );
		}
	}	

}

function loadAndConnect()
{
	ensureServerConnection();
	if( null == ConnectionServer )
	{
		logText( "ERR reaching connection server" );
	}
	if( ConnectionServer.isConnected())
	{
		// get subs - pass our window as a pointer back
		logText( "Calling GAS" );
		ConnectionServer.getAllSubs( window );
		CurrentBucket = "{forceupdate}"; // force update
		
		// select a sub - if new is empty, let's select at index 1...
		
		var targetItem = 0;
		var bucket = null;
		var buckets = ConnectionServer.SubscriptionBuckets;
		if( null != buckets ) bucket = ConnectionServer.SubscriptionBuckets[""];
		if( null == bucket || bucket.length == 0 ) targetItem = 1;
		
		var subsListObj = FindSubsList();
		var items = subsListObj.firstChild.childNodes;
		subsListObj.selectedItem = items[targetItem];
		
		doUpdateSub();
	}
}


/** prefs object */
var prefs = null;

/**
 * 
 */ 
function savePrefsAndReturn()
{
	try
	{
		ensureServerConnection();
		ConnectionServer.clearError();
	}
	catch(e){ alert( "E [102]: " + e ); }
	
	try
	{
		if( null != OptionsDocument )
		{
			var o = null;
			var preferences = new Array();
			if( null == prefs )
				prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
			
			o = OptionsDocument.getElementById( "username" );
			if( null != o ) prefs.setCharPref( "com.pubsub.pubsubsidebar.username", o.value  );
				
			o = OptionsDocument.getElementById( "password" );
			if( null != o ) prefs.setCharPref( "com.pubsub.pubsubsidebar.password", o.value  );
		
			o = OptionsDocument.getElementById( "host" );
			if( null != o ) prefs.setCharPref( "com.pubsub.pubsubsidebar.host", o.value  );
		
			o = OptionsDocument.getElementById( "auto" );
			if( null != o ) prefs.setBoolPref( "com.pubsub.pubsubsidebar.autoconnect", o.checked );

			o = OptionsDocument.getElementById( "closeboxes" );
			if( null != o ) prefs.setBoolPref( "com.pubsub.pubsubsidebar.closeboxes", o.checked );

			o = OptionsDocument.getElementById( "showextended" );
			if( null != o ) prefs.setBoolPref( "com.pubsub.pubsubsidebar.showextended", o.checked );

			o = OptionsDocument.getElementById( "loadhistory" );
			if( null != o ) prefs.setBoolPref( "com.pubsub.pubsubsidebar.loadhistory", o.checked );

			o = OptionsDocument.getElementById( "closeonlink" );
			if( null != o ) prefs.setBoolPref( "com.pubsub.pubsubsidebar.closeonlink", o.checked );

			o = OptionsDocument.getElementById( "showdates" );
			if( null != o ) prefs.setBoolPref( "com.pubsub.pubsubsidebar.showdates", o.checked );

			showSidebarDisplay();
			
		}
	}
	catch( e ){ alert( "E: " + e ); }
}

/**
 * read user prefs; if set, use those values
 */
function setOptionsFields()
{
	try
	{
		ensureServerConnection();
	
		var o = null;
		if( null == prefs )
			prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
	
		var preferences = ConnectionServer.getAccountPrefs();
	
		o = OptionsDocument.getElementById( "username" );
		if( null != o ) o.value = preferences["username"];
			
		o = OptionsDocument.getElementById( "password" );
		if( null != o ) o.value = preferences["password"];
	
		o = OptionsDocument.getElementById( "host" );
		if( null != o ) o.value = preferences["host"];
	
		o = OptionsDocument.getElementById( "auto" );
		if( null != o ) o.checked = preferences["auto"];
		
		o = OptionsDocument.getElementById( "showextended" );
		if( null != o ) o.checked = preferences["showextended"];
		
		o = OptionsDocument.getElementById( "closeboxes" );
		if( null != o ) o.checked = preferences["closeboxes"];

		o = OptionsDocument.getElementById( "loadhistory" );
		if( null != o ) o.checked = preferences["loadhistory"];

		o = OptionsDocument.getElementById( "closeonlink" );
		if( null != o ) o.checked = preferences["closeonlink"];

		o = OptionsDocument.getElementById( "showdates" );
		if( null != o ) o.checked = preferences["showdates"];

	}
	catch(e){ alert( "E: " + e ); }
}

/**
 * enable/disable buttons based on the connection state of the server...
 */
function setButtonStates()
{
	ensureServerConnection();
	var fc = ConnectionServer.isConnected();
	var o = null;
	
	o = ControlsDocument.getElementById( "showConnectButton" );
	if( null != o ) o.style.display = ( fc ? "none" : "inline" );
	
	o = ControlsDocument.getElementById( "showDisconnectButton" );
	if( null != o ) o.style.display = ( fc ? "inline" : "none" );
	
	setStateInfo();

}

/**
 * callback method: state changes
 */
function onStateChange( newState )
{
	setButtonStates();
}

/**
 * onload handler function - this will close the window if it was opened 
 * automatically (not by the menu or toolbar items)
 */
function callLoadFn()
{
	// allowed to open?
	if( !window.parent.fAllowOpen ) 
	{
		window.parent.toggleSidebar('viewPubSubSidebar');		
		return; // probably won't execute
	}

	// unset
	window.parent.fAllowOpen = false;

	// set tb button to normal style
	var obj = window.parent.document.getElementById( "pubsubsidebar-button" );
	if( null != obj ) 
	{
		obj.className = "";
		obj.tooltipText = "PubSub Sidebar";
	}
	
	setDocuments();
	ensureServerConnection();
	
	var preferences = ConnectionServer.getAccountPrefs();
	fCloseBoxes = preferences["closeboxes"];
	fShowExtended = preferences["showextended"];
	fLoadHistory = preferences["loadhistory"];
	fCloseOnLink = preferences["closeonlink"];
	fShowDates = preferences["showdates"];
	
	if( null != DisplayDocument )
	{
		loadAndConnect();
		setButtonStates();
		ConnectionServer.resetPendingCount();
		setHighlightSpanState();
	}
	
	// if this is the options page, then set options 
	// fields from stored preferences

	if( null != OptionsDocument ) setOptionsFields();

}

window.addEventListener( "load", callLoadFn, false );
